{-----------------------------------------------------------------------------
 Unit Name: uApptIterator
 Author:    Rob
 Purpose:   Simple iterator for a list of appointments
 History:
-----------------------------------------------------------------------------}


unit uApptIterator;

interface

uses
  uIApptIterator
  , uAppointments
  , uAppointment
  , sysutils
  ;

type

  EApptIteratorException = class(Exception);

  TApptIterator = class(TInterfacedObject, IApptIterator)
  private

    FApptList: TAppointments;
    FApptNo: integer;

    function First: TAppointment;
    function Last: TAppointment;
    function HasNext: boolean;
    function HasPrev: boolean;
    function Current: TAppointment;
    function Next: TAppointment;
    function Prev: TAppointment;

  public

    constructor Create(ApptList: TAppointments);

  end;

implementation

{ TApptIterator }

constructor TApptIterator.Create(ApptList: TAppointments);
begin
  Assert(Assigned(ApptList), 'ApptList must be assigned');
  inherited Create;
  FApptList := ApptList;
end;

function TApptIterator.Current: TAppointment;
begin
  // Return the appointment we're currently pointing at
  Result := FApptList.Items[FApptNo];
end;

function TApptIterator.First: TAppointment;
begin
  // Go to the first appointment & return it
  FApptNo := 0;
  Result := Current;
end;

function TApptIterator.HasNext: boolean;
begin
  // Have we reached the last appointment yet?
  Result := FApptNo < Pred(FApptList.Count);
end;

function TApptIterator.HasPrev: boolean;
begin
  // Are we at the first appointment?
  Result := FApptNo > 0;
end;

function TApptIterator.Last: TAppointment;
begin
  // Go to the last appointment & return it
  FApptNo := Pred(FApptList.Count);
  Result := Current;
end;

function TApptIterator.Next: TAppointment;
begin
  // If there's another appointment, move to it & return it, otherwise raise an
  // exception
  if HasNext then
  begin
    Inc(FApptNo);
    Result := Current;
  end
  else
  begin
    raise EApptIteratorException.Create('No more appointments');
  end;
end;

function TApptIterator.Prev: TAppointment;
begin
  // If there's a prior appointment, move to it & return it, otherwise raise an
  // exception
  if HasPrev then
  begin
    Dec(FApptNo);
    Result := Current;
  end
  else
  begin
    raise EApptIteratorException.Create('No more appointments');
  end;
end;

end.
